/*
 * Copyright (c) 2009 Anthony Steed and Manuel Fradinho Oliveira
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the 
 * following conditions are met:
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the 
 *    following disclaimer.
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the 
 *    following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of 
 *    this software must display the following acknowledgement:
 *       This product includes software from the book "Networked 
 *       Graphics" written by Anthony Steed and Manuel Fradinho
 *       Oliveira published by Morgan Kaufmann
 * 4. Neither the name of the University nor of the Department
 *    may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 5. Neither the name of Cyntelix may be used to enforese or 
 *    promote products derived from this software without prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO 
 * EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION). HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact Info: Anthony Steed
 *               a.steed@cs.ucl.ac.uk
 *
 *               Manuel Fradinho Oliveira
 *               mfradinho@cyntelix.com
 *
 */
import java.net.*;

public class UDPNetwork extends Network {
  DatagramSocket _sendSocket;
  DatagramSocket _receiveSocket;
  int            _sizeBuf;
  byte[]         _buf;
  InetAddress    _remoteIP;
  DatagramPacket _packet;

  public UDPNetwork (String ip, int local, int remote) {
    super(ip, local,remote);  
    try {
      _sendSocket = new DatagramSocket();
      _receiveSocket = new DatagramSocket(_local);
      _remoteIP = InetAddress.getByName(_ip);
System.err.println(getClass().getName()+"construct: ip to send to : " + _remoteIP);
System.err.println(getClass().getName()+"construct: ip that receives : " + _receiveSocket.getLocalAddress());
System.err.println(getClass().getName()+"construct: ip that receives : port :  " + _receiveSocket.getLocalPort());
    } catch (Exception e) {
      e.printStackTrace();
      throw new RuntimeException(getClass().getName()+": creating the socket");
    }
  }

  public void setRemoteFlock(Flock flock) {
    _remoteFlock = flock;
    _sizeBuf = 128 * flock.numBoids();
    _buf = new byte[_sizeBuf];
    _packet = new DatagramPacket(_buf,_buf.length);
  }

  public void send(StringBuffer buf) {
    DatagramPacket packet = new DatagramPacket(buf.toString().getBytes(),buf.length());
    try {
      packet.setAddress(_remoteIP);
      packet.setPort(_remote);
      _sendSocket.send(packet);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public String recv() {
    String auxStr = null;
    try {
      for (int cycle=0;cycle<_buf.length;cycle++) _buf[cycle] = 0; 
      _receiveSocket.receive(_packet);
      auxStr = new String(_packet.getData());
    } catch (Exception e) {
      e.printStackTrace();
    }
    return auxStr;
  }
  
    public void close() {
    try {  
      _receiveSocket.close();
      _sendSocket.close();
    } catch (Exception e) {
      e.printStackTrace();  
    }
  }
}
